@@ -4,37 +4,32 @@ This module contains the LangChain integrations for Vertex AI service - Google f
 
 **Supported integrations**
 
-1. Google's founational models: Gemini family, Codey, embeddings - `ChatVertexAI`, `VertexAI`, `VertexAIEmbeddings`.
-2. Other Google's foundational models: Imagen - `VertexAIImageCaptioning`, `VertexAIImageCaptioningChat`, `VertexAIImageEditorChat`, `VertexAIImageGeneratorChat`, `VertexAIVisualQnAChat`.
-3. Third-party foundational models available as a an API (mdel-as-a-service) on Vertex Model Garden (Mistral, Llama, Anthropic) - `model_garden.ChatAnthropicVertex`, `model_garden_maas.VertexModelGardenLlama`, `model_garden_maas.VertexModelGardenMistral`.
-4. Third-party foundational models deployed on Vertex AI endpoints from Vertex Model Garden or Hugginface - `VertexAIModelGarden`.
-5. Gemma deployed on Vertex AI endpoints or locally - `GemmaChatLocalHF`, `GemmaChatLocalKaggle`, `GemmaChatVertexAIModelGarden`, `GemmaLocalHF`, `GemmaLocalKaggle`, `GemmaVertexAIModelGarden`.
-5. Vector Search on Vertex AI - `VectorSearchVectorStore`, `VectorSearchVectorStoreDatastore`, `VectorSearchVectorStoreGCS`.
-6. Vertex AI evaluators for generative AI - `VertexPairWiseStringEvaluator`, `VertexStringEvaluator`.
+1. Google's foundational models: ``Gemini`` family, ``Codey``, embeddings - ``ChatVertexAI``, ``VertexAI``, ``VertexAIEmbeddings``.
+2. Other Google's foundational models: Imagen - ``VertexAIImageCaptioning``, ``VertexAIImageCaptioningChat``, ``VertexAIImageEditorChat``, ``VertexAIImageGeneratorChat``, ``VertexAIVisualQnAChat``.
+3. Third-party foundational models available as a an API (mdel-as-a-service) on Vertex Model Garden (Mistral, Llama, Anthropic) - ``model_garden.ChatAnthropicVertex``, ``model_garden_maas.VertexModelGardenLlama``, ``model_garden_maas.VertexModelGardenMistral``.
+4. Third-party foundational models deployed on Vertex AI endpoints from Vertex Model Garden or Huggingface - ``VertexAIModelGarden``.
+5. Gemma deployed on Vertex AI endpoints or locally - ``GemmaChatLocalHF``, ``GemmaChatLocalKaggle``, ``GemmaChatVertexAIModelGarden``, ``GemmaLocalHF``, ``GemmaLocalKaggle``, ``GemmaVertexAIModelGarden``.
+6. Vector Search on Vertex AI - ``VectorSearchVectorStore``, ``VectorSearchVectorStoreDatastore``, ``VectorSearchVectorStoreGCS``.
+7. Vertex AI evaluators for generative AI - ``VertexPairWiseStringEvaluator``, ``VertexStringEvaluator``.
 
 Take a look at detailed documentation for each class for further details.
 
 **Installation**
 
-```bash
-pip install -U langchain-google-vertexai
-```
+.. code-block:: bash
+    pip install langchain-google-vertexai
 
-You need to enable needed Google Cloud APIs (depending on the integration you're using) and set up credentials by either:
-    - Have credentials configured for your environment
-        (gcloud, workload identity, etc...)
-    - Store the path to a service account JSON file as the
-        GOOGLE_APPLICATION_CREDENTIALS environment variable
+You need to enable required Google Cloud APIs (depending on the integration you're using) and set up credentials by either:
+    - Having credentials configured for your environment (gcloud, workload identity, etc...)
+    - Storing the path to a service account JSON file as the ``GOOGLE_APPLICATION_CREDENTIALS`` environment variable
 
-This codebase uses the google.auth library which first looks for the application
+This codebase uses the ``google.auth`` library which first looks for the application
 credentials variable mentioned above, and then looks for system-level auth.
 
-For more information, see:
-https://cloud.google.com/docs/authentication/application-default-credentials#GAC
-and 
-https://googleapis.dev/python/google-auth/latest/reference/google.auth.html#module-google.auth.
-
+**More information:**
 
+- `Credential types <https://cloud.google.com/docs/authentication/application-default-credentials#GAC>`__
+- ``google.auth`` `API reference <https://googleapis.dev/python/google-auth/latest/reference/google.auth.html#module-google.auth>`__
 """  # noqa: E501
 
 from google.cloud.aiplatform_v1beta1.types import (
@@ -48,6 +43,7 @@ from google.cloud.aiplatform_v1beta1.types import (
 from langchain_google_vertexai._enums import (
     HarmBlockThreshold,
     HarmCategory,
+    Modality,
     SafetySetting,
 )
 from langchain_google_vertexai.chains import create_structured_runnable
@@ -102,6 +98,7 @@ __all__ = [
     "GemmaVertexAIModelGarden",
     "HarmBlockThreshold",
     "HarmCategory",
+    "Modality",
     "PydanticFunctionsOutputParser",
     "SafetySetting",
     "Schema",
