@@ -2,7 +2,18 @@ from __future__ import annotations
 
 import re
 from concurrent.futures import Executor
-from typing import Any, Callable, ClassVar, Dict, List, Optional, Sequence, Tuple
+from typing import (
+    Any,
+    Callable,
+    ClassVar,
+    Dict,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+    cast,
+)
 
 import vertexai  # type: ignore[import-untyped]
 from google.api_core.client_options import ClientOptions
@@ -13,35 +24,38 @@ from google.cloud.aiplatform.gapic import (
     PredictionServiceClient,
 )
 from google.cloud.aiplatform.models import Prediction
+from google.cloud.aiplatform_v1.services.prediction_service import (
+    PredictionServiceAsyncClient as v1PredictionServiceAsyncClient,
+)
+from google.cloud.aiplatform_v1.services.prediction_service import (
+    PredictionServiceClient as v1PredictionServiceClient,
+)
 from google.cloud.aiplatform_v1beta1.services.prediction_service import (
     PredictionServiceAsyncClient as v1beta1PredictionServiceAsyncClient,
 )
 from google.cloud.aiplatform_v1beta1.services.prediction_service import (
     PredictionServiceClient as v1beta1PredictionServiceClient,
 )
+from google.cloud.aiplatform_v1beta1.types.content import Modality
 from google.protobuf import json_format
 from google.protobuf.struct_pb2 import Value
 from langchain_core.outputs import Generation, LLMResult
 from pydantic import BaseModel, ConfigDict, Field, model_validator
-from typing_extensions import Self
+from typing_extensions import Literal, Self
 from vertexai.generative_models._generative_models import (  # type: ignore
     SafetySettingsType,
 )
-from vertexai.language_models import (  # type: ignore[import-untyped]
-    TextGenerationModel,
-)
 
+from langchain_google_vertexai._client_utils import (
+    _get_async_prediction_client,
+    _get_client_options,
+    _get_prediction_client,
+)
 from langchain_google_vertexai._utils import (
-    GoogleModelFamily,
     get_client_info,
     get_user_agent,
-    is_gemini_model,
 )
 
-_PALM_DEFAULT_MAX_OUTPUT_TOKENS = TextGenerationModel._DEFAULT_MAX_OUTPUT_TOKENS
-_PALM_DEFAULT_TEMPERATURE = 0.0
-_PALM_DEFAULT_TOP_P = 0.95
-_PALM_DEFAULT_TOP_K = 40
 _DEFAULT_LOCATION = "us-central1"
 
 
@@ -87,6 +101,11 @@ class _VertexAIBase(BaseModel):
     "The default custom credentials (google.auth.credentials.Credentials) to use "
     "when making API calls. If not provided, credentials will be ascertained from "
     "the environment."
+    endpoint_version: Literal["v1", "v1beta1"] = "v1beta1"
+    """Whether to use v1 or v1beta1 endpoint.
+    
+    v1 is more performant, but v1beta1 might have some new features.
+    """
 
     model_config = ConfigDict(
         populate_by_name=True,
@@ -103,15 +122,18 @@ class _VertexAIBase(BaseModel):
             values["api_transport"] = initializer.global_config._api_transport
         if "location" not in values:
             values["location"] = initializer.global_config.location
-        if values.get("api_endpoint"):
-            api_endpoint = values["api_endpoint"]
+        if values.get("api_endpoint") or values.get("base_url"):
+            api_endpoint = values.get("api_endpoint", values.get("base_url"))
         else:
             location = values.get("location", cls.model_fields["location"].default)
-            api_endpoint = f"{location}-{constants.PREDICTION_API_BASE_PATH}"
-        client_options = ClientOptions(api_endpoint=api_endpoint)
-        if values.get("client_cert_source"):
-            client_options.client_cert_source = values["client_cert_source"]
-        values["client_options"] = client_options
+            api_endpoint = (
+                f"{'' if location == 'global' else location + '-'}"
+                f"{constants.PREDICTION_API_BASE_PATH}"
+            )
+        values["client_options"] = _get_client_options(
+            api_endpoint=api_endpoint,
+            cert_source=values.get("client_cert_source"),
+        )
         additional_headers = values.get("additional_headers") or {}
         values["default_metadata"] = tuple(additional_headers.items())
         return values
@@ -126,34 +148,32 @@ class _VertexAIBase(BaseModel):
         return self
 
     @property
-    def prediction_client(self) -> v1beta1PredictionServiceClient:
+    def prediction_client(
+        self,
+    ) -> Union[v1beta1PredictionServiceClient, v1PredictionServiceClient]:
         """Returns PredictionServiceClient."""
         if self.client is None:
-            self.client = v1beta1PredictionServiceClient(
+            self.client = _get_prediction_client(
+                endpoint_version=self.endpoint_version,
                 credentials=self.credentials,
                 client_options=self.client_options,
-                client_info=get_client_info(module=self._user_agent),
                 transport=self.api_transport,
+                user_agent=self._user_agent,
             )
         return self.client
 
     @property
-    def async_prediction_client(self) -> v1beta1PredictionServiceAsyncClient:
+    def async_prediction_client(
+        self,
+    ) -> Union[v1PredictionServiceAsyncClient, v1beta1PredictionServiceAsyncClient]:
         """Returns PredictionServiceClient."""
         if self.async_client is None:
-            async_client_kwargs: dict[str, Any] = dict(
-                client_options=self.client_options,
-                client_info=get_client_info(module=self._user_agent),
+            self.async_client = _get_async_prediction_client(
+                endpoint_version=self.endpoint_version,
                 credentials=self.credentials,
+                client_options=cast(ClientOptions, self.client_options),
+                user_agent=self._user_agent,
             )
-
-            if self.api_transport is not None:
-                async_client_kwargs["transport"] = self.api_transport
-
-            self.async_client = v1beta1PredictionServiceAsyncClient(
-                **async_client_kwargs
-            )
-
         return self.async_client
 
     @property
@@ -175,6 +195,12 @@ class _VertexAICommon(_VertexAIBase):
     "Underlying model name."
     temperature: Optional[float] = None
     "Sampling temperature, it controls the degree of randomness in token selection."
+    frequency_penalty: Optional[float] = None
+    "Positive values penalize tokens that repeatedly appear in the generated text, "
+    "decreasing the probability of repeating content."
+    presence_penalty: Optional[float] = None
+    "Positive values penalize tokens that already appear in the generated text, "
+    "increasing the probability of generating more diverse content."
     max_output_tokens: Optional[int] = Field(default=None, alias="max_tokens")
     "Token limit determines the maximum amount of text output from one prompt."
     top_p: Optional[float] = None
@@ -189,7 +215,6 @@ class _VertexAICommon(_VertexAIBase):
     """Random seed for the generation."""
     streaming: bool = False
     """Whether to stream the results or not."""
-    model_family: Optional[GoogleModelFamily] = None  #: :meta private:
     safety_settings: Optional["SafetySettingsType"] = None
     """The default safety settings to use for all generations. 
     
@@ -207,13 +232,23 @@ class _VertexAICommon(_VertexAIBase):
             """  # noqa: E501
 
     tuned_model_name: Optional[str] = None
-    """The name of a tuned model. If tuned_model_name is passed
-    model_name will be used to determine the model family
-    """
+    """The name of a tuned model."""
 
-    @property
-    def _is_gemini_model(self) -> bool:
-        return is_gemini_model(self.model_family)  # type: ignore[arg-type]
+    response_modalities: Optional[List[Modality]] = Field(
+        default=None, description=("A list of modalities of the response")
+    )
+
+    thinking_budget: Optional[int] = Field(
+        default=None, description="Indicates the thinking budget in tokens."
+    )
+    include_thoughts: Optional[bool] = Field(
+        default=None,
+        description="Indicates whether to include thoughts in the response.",
+    )
+    audio_timestamp: Optional[bool] = Field(
+        default=None,
+        description="Enable timestamp understanding of audio-only files",
+    )
 
     @property
     def _llm_type(self) -> str:
@@ -230,30 +265,15 @@ class _VertexAICommon(_VertexAIBase):
 
     @property
     def _default_params(self) -> Dict[str, Any]:
-        if self.model_family == GoogleModelFamily.GEMINI:
-            default_params: Dict[str, Any] = {}
-        elif self.model_family == GoogleModelFamily.GEMINI_ADVANCED:
-            default_params = {}
-        else:
-            default_params = {
-                "temperature": _PALM_DEFAULT_TEMPERATURE,
-                "max_output_tokens": _PALM_DEFAULT_MAX_OUTPUT_TOKENS,
-                "top_p": _PALM_DEFAULT_TOP_P,
-                "top_k": _PALM_DEFAULT_TOP_K,
-            }
+        default_params: Dict[str, Any] = {}
         params = {
             "temperature": self.temperature,
             "max_output_tokens": self.max_output_tokens,
             "candidate_count": self.n,
             "seed": self.seed,
+            "top_k": self.top_k,
+            "top_p": self.top_p,
         }
-        if not self.model_family == GoogleModelFamily.CODEY:
-            params.update(
-                {
-                    "top_k": self.top_k,
-                    "top_p": self.top_p,
-                }
-            )
         updated_params = {}
         for param_name, param_value in params.items():
             default_value = default_params.get(param_name)
@@ -269,7 +289,9 @@ class _VertexAICommon(_VertexAIBase):
             project=values.get("project"),
             location=values.get("location"),
             credentials=values.get("credentials"),
-            api_transport=values.get("api_transport"),
+            # if both project and api_transport are empty, vertexai.init() sets
+            # the default transport to "rest"
+            api_transport=values.get("api_transport", "grpc"),
             api_endpoint=values.get("api_endpoint"),
             request_metadata=values.get("default_metadata"),
         )
