@@ -1,9 +1,8 @@
 """Utilities to init Vertex AI."""
 
-import dataclasses
 import math
+import os
 import re
-from enum import Enum, auto
 from importlib import metadata
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
@@ -15,7 +14,6 @@ from langchain_core.callbacks import (
     AsyncCallbackManagerForLLMRun,
     CallbackManagerForLLMRun,
 )
-from langchain_core.language_models.llms import create_base_retry_decorator
 from vertexai.generative_models import (  # type: ignore[import-untyped]
     Candidate,
     Image,
@@ -24,6 +22,11 @@ from vertexai.language_models import (  # type: ignore[import-untyped]
     TextGenerationResponse,
 )
 
+from langchain_google_vertexai._retry import create_base_retry_decorator
+
+_TELEMETRY_TAG = "remote_reasoning_engine"
+_TELEMETRY_ENV_VARIABLE_NAME = "GOOGLE_CLOUD_AGENT_ENGINE_ID"
+
 
 def create_retry_decorator(
     *,
@@ -31,8 +34,22 @@ def create_retry_decorator(
     run_manager: Optional[
         Union[AsyncCallbackManagerForLLMRun, CallbackManagerForLLMRun]
     ] = None,
+    wait_exponential_kwargs: Optional[dict[str, float]] = None,
 ) -> Callable[[Any], Any]:
-    """Creates a retry decorator for Vertex / Palm LLMs."""
+    """Creates a retry decorator for Vertex / Palm LLMs.
+
+    Args:
+        max_retries: Number of retries. Default is 1.
+        run_manager: Callback manager for the run. Default is None.
+        wait_exponential_kwargs: Optional dictionary with parameters:
+            - multiplier: Initial wait time multiplier (default: 1.0)
+            - min: Minimum wait time in seconds (default: 4.0)
+            - max: Maximum wait time in seconds (default: 10.0)
+            - exp_base: Exponent base to use (default: 2.0)
+
+    Returns:
+        A retry decorator.
+    """
 
     errors = [
         google.api_core.exceptions.ResourceExhausted,
@@ -42,7 +59,10 @@ def create_retry_decorator(
         google.api_core.exceptions.GoogleAPIError,
     ]
     decorator = create_base_retry_decorator(
-        error_types=errors, max_retries=max_retries, run_manager=run_manager
+        error_types=errors,
+        max_retries=max_retries,
+        run_manager=run_manager,
+        wait_exponential_kwargs=wait_exponential_kwargs,
     )
     return decorator
 
@@ -77,6 +97,8 @@ def get_user_agent(module: Optional[str] = None) -> Tuple[str, str]:
     client_library_version = (
         f"{langchain_version}-{module}" if module else langchain_version
     )
+    if os.environ.get(_TELEMETRY_ENV_VARIABLE_NAME):
+        client_library_version += f"+{_TELEMETRY_TAG}"
     return client_library_version, f"langchain-google-vertexai/{client_library_version}"
 
 
@@ -116,161 +138,94 @@ def load_image_from_gcs(path: str, project: Optional[str] = None) -> Image:
     return Image.from_bytes(blobs[0].download_as_bytes())
 
 
-class GoogleModelFamily(str, Enum):
-    GEMINI = auto()
-    GEMINI_ADVANCED = auto()
-    CODEY = auto()
-    PALM = auto()
-
-    @classmethod
-    def _missing_(cls, value: Any) -> "GoogleModelFamily":
-        # https://cloud.google.com/vertex-ai/generative-ai/docs/learn/model-versioning
-        model_name = value.lower()
-        gemini_advanced_models = {
-            "medlm-large-1.5-preview",
-            "medlm-large-1.5-001",
-            "medlm-large-1.5@001",
-        }
-        if (
-            "gemini-1.5" in model_name
-            or model_name in gemini_advanced_models
-            or "gemini-2" in model_name
-        ):
-            return GoogleModelFamily.GEMINI_ADVANCED
-        if "gemini" in model_name:
-            return GoogleModelFamily.GEMINI
-        if "code" in model_name:
-            return GoogleModelFamily.CODEY
-        if "medlm-medium@latest" in model_name:
-            return GoogleModelFamily.GEMINI
-        if "bison" in model_name or "medlm" in model_name:
-            return GoogleModelFamily.PALM
-        return GoogleModelFamily.GEMINI
-
-
-def is_gemini_model(model_family: GoogleModelFamily) -> bool:
-    """Returns True if the model name is a Gemini model."""
-    return model_family in [GoogleModelFamily.GEMINI, GoogleModelFamily.GEMINI_ADVANCED]
-
-
 def get_generation_info(
     candidate: Union[TextGenerationResponse, Candidate],
-    is_gemini: bool,
     *,
     stream: bool = False,
     usage_metadata: Optional[Dict] = None,
     logprobs: Union[bool, int] = False,
 ) -> Dict[str, Any]:
-    if is_gemini:
-        # https://cloud.google.com/vertex-ai/docs/generative-ai/model-reference/gemini#response_body
-        info = {
-            "is_blocked": any([rating.blocked for rating in candidate.safety_ratings]),
-            "safety_ratings": [
+    # https://cloud.google.com/vertex-ai/docs/generative-ai/model-reference/gemini#response_body
+    info = {
+        "is_blocked": any([rating.blocked for rating in candidate.safety_ratings]),
+        "safety_ratings": [
+            {
+                "category": rating.category.name,
+                "probability_label": rating.probability.name,
+                "probability_score": rating.probability_score,
+                "blocked": rating.blocked,
+                "severity": rating.severity.name,
+                "severity_score": rating.severity_score,
+            }
+            # Image generation models sometime return ratings that are not
+            # included in the proto.
+            for rating in candidate.safety_ratings
+            if hasattr(rating.category, "name")
+        ],
+        "citation_metadata": (
+            proto.Message.to_dict(candidate.citation_metadata)
+            if candidate.citation_metadata
+            else None
+        ),
+        "usage_metadata": usage_metadata,
+        "finish_reason": (
+            candidate.finish_reason.name if candidate.finish_reason else None
+        ),
+        "finish_message": (
+            candidate.finish_message if candidate.finish_message else None
+        ),
+    }
+    if hasattr(candidate, "avg_logprobs") and candidate.avg_logprobs is not None:
+        if (
+            isinstance(candidate.avg_logprobs, float)
+            and not math.isnan(candidate.avg_logprobs)
+            and candidate.avg_logprobs < 0
+        ):
+            info["avg_logprobs"] = candidate.avg_logprobs
+
+    if hasattr(candidate, "logprobs_result") and logprobs:
+
+        def is_valid_logprob(prob):
+            return isinstance(prob, float) and not math.isnan(prob) and prob < 0
+
+        chosen_candidates = candidate.logprobs_result.chosen_candidates
+        top_candidates_list = candidate.logprobs_result.top_candidates
+        logprobs_int = 0 if logprobs is True else logprobs
+
+        valid_log_probs = []
+        for i, chosen in enumerate(chosen_candidates):
+            if not is_valid_logprob(chosen.log_probability):
+                continue
+
+            top_logprobs = []
+            if logprobs_int > 0:
+                for top in top_candidates_list[i].candidates[:logprobs_int]:
+                    if not is_valid_logprob(top.log_probability):
+                        continue
+                    top_logprobs.append(
+                        {"token": top.token, "logprob": top.log_probability}
+                    )
+
+            valid_log_probs.append(
                 {
-                    "category": rating.category.name,
-                    "probability_label": rating.probability.name,
-                    "probability_score": rating.probability_score,
-                    "blocked": rating.blocked,
-                    "severity": rating.severity.name,
-                    "severity_score": rating.severity_score,
+                    "token": chosen.token,
+                    "logprob": chosen.log_probability,
+                    "top_logprobs": top_logprobs,
                 }
-                for rating in candidate.safety_ratings
-            ],
-            "citation_metadata": (
-                proto.Message.to_dict(candidate.citation_metadata)
-                if candidate.citation_metadata
-                else None
-            ),
-            "usage_metadata": usage_metadata,
-            "finish_reason": (
-                candidate.finish_reason.name if candidate.finish_reason else None
-            ),
-            "finish_message": (
-                candidate.finish_message if candidate.finish_message else None
-            ),
-        }
-        if hasattr(candidate, "avg_logprobs") and candidate.avg_logprobs is not None:
-            if (
-                isinstance(candidate.avg_logprobs, float)
-                and not math.isnan(candidate.avg_logprobs)
-                and candidate.avg_logprobs < 0
-            ):
-                info["avg_logprobs"] = candidate.avg_logprobs
+            )
 
-        if hasattr(candidate, "logprobs_result") and logprobs:
+        if valid_log_probs:
+            info["logprobs_result"] = valid_log_probs
 
-            def is_valid_logprob(prob):
-                return isinstance(prob, float) and not math.isnan(prob) and prob < 0
-
-            chosen_candidates = candidate.logprobs_result.chosen_candidates
-            top_candidates_list = candidate.logprobs_result.top_candidates
-            logprobs_int = 0 if logprobs is True else logprobs
-
-            valid_log_probs = []
-            for i, chosen in enumerate(chosen_candidates):
-                if not is_valid_logprob(chosen.log_probability):
-                    continue
-
-                top_logprobs = []
-                if logprobs_int > 0:
-                    for top in top_candidates_list[i].candidates[:logprobs_int]:
-                        if not is_valid_logprob(top.log_probability):
-                            continue
-                        top_logprobs.append(
-                            {"token": top.token, "logprob": top.log_probability}
-                        )
-
-                valid_log_probs.append(
-                    {
-                        "token": chosen.token,
-                        "logprob": chosen.log_probability,
-                        "top_logprobs": top_logprobs,
-                    }
-                )
-
-            if valid_log_probs:
-                info["logprobs_result"] = valid_log_probs
-
-        try:
-            if candidate.grounding_metadata:
-                info["grounding_metadata"] = proto.Message.to_dict(
-                    candidate.grounding_metadata
-                )
-        except AttributeError:
-            pass
-        info = {k: v for k, v in info.items() if v is not None}
+    try:
+        if candidate.grounding_metadata:
+            info["grounding_metadata"] = proto.Message.to_dict(
+                candidate.grounding_metadata
+            )
+    except AttributeError:
+        pass
+    info = {k: v for k, v in info.items() if v is not None}
     # https://cloud.google.com/vertex-ai/docs/generative-ai/model-reference/text-chat#response_body
-    else:
-        info = dataclasses.asdict(candidate)
-        info.pop("text")
-        info = {k: v for k, v in info.items() if not k.startswith("_")}
-        if usage_metadata:
-            info_usage_metadata = {}
-            output_usage = usage_metadata.get("tokenMetadata", {}).get(
-                "outputTokenCount", {}
-            )
-            info_usage_metadata["candidates_billable_characters"] = output_usage.get(
-                "totalBillableCharacters"
-            )
-            info_usage_metadata["candidates_token_count"] = output_usage.get(
-                "totalTokens"
-            )
-            input_usage = usage_metadata.get("tokenMetadata", {}).get(
-                "inputTokenCount", {}
-            )
-            info_usage_metadata["prompt_billable_characters"] = input_usage.get(
-                "totalBillableCharacters"
-            )
-            info_usage_metadata["prompt_token_count"] = input_usage.get("totalTokens")
-            info["usage_metadata"] = {k: v for k, v in info_usage_metadata.items() if v}
-
-            # NOTE:
-            # "safety_attributes" can contain different values for the same keys
-            # for each generation. Put it in a list, so it can be merged later by
-            # merge_dicts().
-            #
-            safety_attributes = info.get("safety_attributes") or {}
-            info["safety_attributes"] = [safety_attributes]
 
     if stream:
         # Remove non-streamable types, like bools.
@@ -339,3 +294,46 @@ def _get_def_key_from_schema_path(schema_path: str) -> str:
         raise ValueError(error_message)
 
     return parts[-1]
+
+
+def _strip_nullable_anyof(schema: dict[str, Any]) -> dict[str, Any]:
+    """Collapse ``anyOf([{...}, {"type": "null"}])``` into the non-null schema,
+    leave the rest of the keywords alone, and make the property optional.
+    Works in place.
+    """
+
+    def walk(node):
+        if not isinstance(node, dict):
+            return
+
+        props = node.get("properties", {})
+        for prop_name, prop_schema in list(props.items()):
+            any_of = prop_schema.get("anyOf")
+            if any_of and len(any_of) == 2:
+                null_branch = next((b for b in any_of if b.get("type") == "null"), None)
+                other_branch = next((b for b in any_of if b is not null_branch), None)
+
+                if null_branch and other_branch:
+                    # remove the anyOf *only*
+                    prop_schema.pop("anyOf")
+                    # and overlay the surviving branch
+                    prop_schema.update(other_branch)
+
+                    # make the property optional
+                    req = node.get("required", [])
+                    if prop_name in req:
+                        req.remove(prop_name)
+                        if not req:
+                            node.pop("required")
+
+            walk(prop_schema)
+
+        if "items" in node:
+            walk(node["items"])
+
+        for combiner in ("allOf", "anyOf", "oneOf"):
+            for sub in node.get(combiner, []):
+                walk(sub)
+
+    walk(schema)
+    return schema
