@@ -18,6 +18,9 @@ from typing import (
 
 import google.cloud.aiplatform_v1beta1.types as gapic
 import vertexai.generative_models as vertexai  # type: ignore
+from google.cloud.aiplatform_v1beta1.types import (
+    ToolConfig as GapicToolConfig,
+)
 from langchain_core.exceptions import OutputParserException
 from langchain_core.output_parsers import BaseOutputParser
 from langchain_core.outputs import ChatGeneration, Generation
@@ -29,6 +32,7 @@ from langchain_core.utils.function_calling import (
 )
 from langchain_core.utils.json_schema import dereference_refs
 from pydantic import BaseModel
+from typing_extensions import NotRequired
 
 logger = logging.getLogger(__name__)
 
@@ -44,13 +48,17 @@ _GoogleSearchRetrievalLike = Union[
     gapic.GoogleSearchRetrieval,
     Dict[str, Any],
 ]
+_GoogleSearchLike = Union[gapic.Tool.GoogleSearch, Dict[str, Any]]
 _RetrievalLike = Union[gapic.Retrieval, Dict[str, Any]]
+_CodeExecutionLike = Union[gapic.Tool.CodeExecution, Dict[str, Any]]
 
 
 class _ToolDictLike(TypedDict):
     function_declarations: Optional[List[_FunctionDeclarationLike]]
     google_search_retrieval: Optional[_GoogleSearchRetrievalLike]
+    google_search: Optional[_GoogleSearchLike]
     retrieval: Optional[_RetrievalLike]
+    code_execution: NotRequired[_CodeExecutionLike]
 
 
 _ToolType = Union[gapic.Tool, vertexai.Tool, _ToolDictLike, _FunctionDeclarationLike]
@@ -109,7 +117,7 @@ def _format_json_schema_to_gapic(
     """Format a JSON schema from a Pydantic V2 BaseModel to gapic."""
     converted_schema: Dict[str, Any] = {}
     for key, value in schema.items():
-        if key == "definitions":
+        if key == "$defs":
             continue
         elif key == "items":
             converted_schema["items"] = _format_json_schema_to_gapic(
@@ -144,6 +152,12 @@ def _format_json_schema_to_gapic(
                 if required_fields and parent_key in required_fields:
                     required_fields.remove(parent_key)
                 continue
+            converted_schema["anyOf"] = [
+                _format_json_schema_to_gapic(
+                    anyOf_type, "anyOf", schema.get("required", [])
+                )
+                for anyOf_type in value
+            ]
         elif key not in _ALLOWED_SCHEMA_FIELDS_SET:
             logger.warning(f"Key '{key}' is not supported in schema, ignoring")
         else:
@@ -154,7 +168,10 @@ def _format_json_schema_to_gapic(
 def _dict_to_gapic_schema(
     schema: Dict[str, Any], pydantic_version: str = "v1"
 ) -> gapic.Schema:
+    # Resolve refs in schema because $refs and $defs are not supported
+    # by the Gemini API.
     dereferenced_schema = dereference_refs(schema)
+
     if pydantic_version == "v1":
         formatted_schema = _format_json_schema_to_gapic_v1(dereferenced_schema)
     else:
@@ -216,16 +233,27 @@ def _format_pydantic_to_function_declaration(
 def _format_dict_to_function_declaration(
     tool: Union[FunctionDescription, Dict[str, Any]],
 ) -> gapic.FunctionDeclaration:
+    pydantic_version_v2 = False
+
     # Ensure we send "anyOf" parameters through pydantic v2 schema parsing
-    pydantic_version = None
-    if isinstance(tool, dict):
-        properties = tool.get("parameters", {}).get("properties", {}).values()
+    def _check_v2(parameters):
+        properties = parameters.get("properties", {}).values()
         for property in properties:
             if "anyOf" in property:
-                pydantic_version = "v2"
-    if pydantic_version:
+                return True
+            if "parameters" in property:
+                if _check_v2(property["parameters"]):
+                    return True
+            if "items" in property:
+                if _check_v2(property["items"]):
+                    return True
+        return False
+
+    if isinstance(tool, dict):
+        pydantic_version_v2 = _check_v2(tool.get("parameters", {}))
+    if pydantic_version_v2:
         parameters = _dict_to_gapic_schema(
-            tool.get("parameters", {}), pydantic_version=pydantic_version
+            tool.get("parameters", {}), pydantic_version="v2"
         )
     else:
         parameters = _dict_to_gapic_schema(tool.get("parameters", {}))
@@ -289,6 +317,8 @@ def _format_to_gapic_tool(tools: _ToolsType) -> gapic.Tool:
                 gapic_tool.function_declarations.extend(rt.function_declarations)
             if "google_search" in rt:
                 gapic_tool.google_search = rt.google_search
+            if "code_execution" in rt:
+                gapic_tool.code_execution = rt.code_execution
         elif isinstance(tool, dict):
             # not _ToolDictLike
             if not any(
@@ -296,7 +326,9 @@ def _format_to_gapic_tool(tools: _ToolsType) -> gapic.Tool:
                 for f in [
                     "function_declarations",
                     "google_search_retrieval",
+                    "google_search",
                     "retrieval",
+                    "code_execution",
                 ]
             ):
                 fd = _format_to_gapic_function_declaration(tool)
@@ -321,8 +353,16 @@ def _format_to_gapic_tool(tools: _ToolsType) -> gapic.Tool:
                 gapic_tool.google_search_retrieval = gapic.GoogleSearchRetrieval(
                     tool["google_search_retrieval"]
                 )
+            if "google_search" in tool:
+                gapic_tool.google_search = gapic.Tool.GoogleSearch(
+                    tool["google_search"]
+                )
             if "retrieval" in tool:
                 gapic_tool.retrieval = gapic.Retrieval(tool["retrieval"])
+            if "code_execution" in tool:
+                gapic_tool.code_execution = gapic.Tool.CodeExecution(
+                    tool["code_execution"]
+                )
         else:
             fd = _format_to_gapic_function_declaration(tool)
             gapic_tool.function_declarations.append(fd)
@@ -424,7 +464,7 @@ def _format_tool_config(tool_config: _ToolConfigDict) -> Union[gapic.ToolConfig,
 def _tool_choice_to_tool_config(
     tool_choice: _ToolChoiceType,
     all_names: List[str],
-) -> _ToolConfigDict:
+) -> Optional[GapicToolConfig]:
     allowed_function_names: Optional[List[str]] = None
     if tool_choice is True or tool_choice == "any":
         mode = gapic.FunctionCallingConfig.Mode.ANY
@@ -448,6 +488,14 @@ def _tool_choice_to_tool_config(
             allowed_function_names = tool_choice["function_calling_config"].get(
                 "allowed_function_names"
             )
+        elif (
+            "type" in tool_choice
+            and tool_choice["type"] == "function"
+            and "function" in tool_choice
+            and "name" in tool_choice["function"]
+        ):
+            mode = gapic.FunctionCallingConfig.Mode.ANY
+            allowed_function_names = [tool_choice["function"]["name"]]
         else:
             raise ValueError(
                 f"Unrecognized tool choice format:\n\n{tool_choice=}\n\nShould match "
@@ -455,9 +503,10 @@ def _tool_choice_to_tool_config(
             )
     else:
         raise ValueError(f"Unrecognized tool choice format:\n\n{tool_choice=}")
-    return _ToolConfigDict(
+    tool_config = _ToolConfigDict(
         function_calling_config=_FunctionCallingConfigDict(
             mode=mode,
             allowed_function_names=allowed_function_names,
         )
     )
+    return _format_tool_config(tool_config)
