@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+import logging
+from difflib import get_close_matches
 from typing import Any, AsyncIterator, Dict, Iterator, List, Optional
 
 from langchain_core.callbacks.manager import (
@@ -12,9 +14,11 @@ from langchain_core.outputs import Generation, GenerationChunk, LLMResult
 from pydantic import ConfigDict, Field, model_validator
 from typing_extensions import Self
 
-from langchain_google_vertexai._base import GoogleModelFamily, _VertexAICommon
+from langchain_google_vertexai._base import _VertexAICommon
 from langchain_google_vertexai.chat_models import ChatVertexAI
 
+logger = logging.getLogger(__name__)
+
 
 class VertexAI(_VertexAICommon, BaseLLM):
     """Google Vertex AI large language models."""
@@ -25,11 +29,44 @@ class VertexAI(_VertexAICommon, BaseLLM):
     """The name of a tuned model. If tuned_model_name is passed
     model_name will be used to determine the model family
     """
+    response_mime_type: Optional[str] = None
+    """Optional. Output response mimetype of the generated candidate text. Only
+        supported in Gemini 1.5 and later models. Supported mimetype:
+            * "text/plain": (default) Text output.
+            * "application/json": JSON response in the candidates.
+            * "text/x.enum": Enum in plain text.
+       The model also needs to be prompted to output the appropriate response
+       type, otherwise the behavior is undefined. This is a preview feature.
+    """
+    response_schema: Optional[Dict[str, Any]] = None
+    """ Optional. Enforce an schema to the output.
+        The format of the dictionary should follow Open API schema.
+    """
 
     def __init__(self, *, model_name: Optional[str] = None, **kwargs: Any) -> None:
-        """Needed for mypy typing to recognize model_name as a valid arg."""
+        """Needed for mypy typing to recognize model_name as a valid arg
+        and for arg validation.
+        """
         if model_name:
             kwargs["model_name"] = model_name
+
+        # Get all valid field names, including aliases
+        valid_fields = set()
+        for field_name, field_info in self.__class__.model_fields.items():
+            valid_fields.add(field_name)
+            if hasattr(field_info, "alias") and field_info.alias is not None:
+                valid_fields.add(field_info.alias)
+
+        # Check for unrecognized arguments
+        for arg in kwargs:
+            if arg not in valid_fields:
+                suggestions = get_close_matches(arg, valid_fields, n=1)
+                suggestion = (
+                    f" Did you mean: '{suggestions[0]}'?" if suggestions else ""
+                )
+                logger.warning(
+                    f"Unexpected argument '{arg}' provided to VertexAI.{suggestion}"
+                )
         super().__init__(**kwargs)
 
     model_config = ConfigDict(
@@ -48,12 +85,6 @@ class VertexAI(_VertexAICommon, BaseLLM):
     @model_validator(mode="after")
     def validate_environment(self) -> Self:
         """Validate that the python package exists in environment."""
-        if "medlm" in self.model_name and self.model_family == GoogleModelFamily.PALM:
-            err = (
-                "MedLM on Palm is not supported any more! Please, use Gemini or "
-                "switch to langchain-google-vertexai==2.0.13"
-            )
-            raise ValueError(err)
         self.client = ChatVertexAI(
             model_name=self.model_name,
             tuned_model_name=self.tuned_model_name,
@@ -70,6 +101,8 @@ class VertexAI(_VertexAICommon, BaseLLM):
             safety_settings=self.safety_settings,
             n=self.n,
             seed=self.seed,
+            response_schema=self.response_schema,
+            response_mime_type=self.response_mime_type,
         )
         return self
 
