@@ -33,7 +33,6 @@ from langchain_core.messages import (
     AIMessage,
     BaseMessage,
 )
-from langchain_core.messages.ai import UsageMetadata
 from langchain_core.outputs import (
     ChatGeneration,
     ChatGenerationChunk,
@@ -55,28 +54,25 @@ from langchain_google_vertexai._anthropic_parsers import (
     _extract_tool_calls,
 )
 from langchain_google_vertexai._anthropic_utils import (
+    _create_usage_metadata,
+    _documents_in_params,
     _format_messages_anthropic,
     _make_message_chunk_from_anthropic_event,
+    _thinking_in_params,
     _tools_in_params,
     convert_to_anthropic_tool,
 )
 from langchain_google_vertexai._base import _BaseVertexAIModelGarden, _VertexAICommon
-from langchain_google_vertexai.utils import create_base_retry_decorator
-
-
-class CacheUsageMetadata(UsageMetadata):
-    cache_creation_input_tokens: Optional[int]
-    """The number of input tokens used to create the cache entry."""
-    cache_read_input_tokens: Optional[int]
-    """The number of input tokens read from the cache."""
+from langchain_google_vertexai._retry import create_base_retry_decorator
 
 
 def _create_retry_decorator(
-    llm: ChatAnthropicVertex,
     *,
+    max_retries: int = 3,
     run_manager: Optional[
         Union[AsyncCallbackManagerForLLMRun, CallbackManagerForLLMRun]
     ] = None,
+    wait_exponential_kwargs: Optional[dict[str, float]] = None,
 ) -> Callable[[Any], Any]:
     """Creates a retry decorator for Anthropic Vertex LLMs with proper tracing."""
     from anthropic import (  # type: ignore[unused-ignore, import-not-found]
@@ -92,7 +88,10 @@ def _create_retry_decorator(
     ]
 
     return create_base_retry_decorator(
-        error_types=errors, max_retries=llm.max_retries, run_manager=run_manager
+        error_types=errors,
+        max_retries=max_retries,
+        run_manager=run_manager,
+        wait_exponential_kwargs=wait_exponential_kwargs,
     )
 
 
@@ -173,6 +172,14 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
     max_retries: int = Field(
         default=3, description="Number of retries for error handling."
     )
+    wait_exponential_kwargs: Optional[dict[str, float]] = Field(
+        default=None,
+        description="Optional dictionary with parameters for wait_exponential: "
+        "- multiplier: Initial wait time multiplier (default: 1.0) "
+        "- min: Minimum wait time in seconds (default: 4.0) "
+        "- max: Maximum wait time in seconds (default: 10.0) "
+        "- exp_base: Exponent base to use (default: 2.0) ",
+    )
     timeout: Optional[Union[float, httpx.Timeout]] = Field(
         default=None,
         description="Timeout for API requests.",
@@ -181,6 +188,7 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
     model_config = ConfigDict(
         populate_by_name=True,
     )
+    model_kwargs: dict[str, Any] = Field(default_factory=dict)
 
     # Needed so that mypy doesn't flag missing aliased init args.
     def __init__(self, **kwargs: Any) -> None:
@@ -202,6 +210,7 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
         self.client = AnthropicVertex(
             project_id=project_id,
             region=self.location,
+            base_url=self.api_endpoint,
             max_retries=0,
             access_token=self.access_token,
             credentials=self.credentials,
@@ -210,6 +219,7 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
         self.async_client = AsyncAnthropicVertex(
             project_id=project_id,
             region=self.location,
+            base_url=self.api_endpoint,
             max_retries=0,
             access_token=self.access_token,
             credentials=self.credentials,
@@ -219,13 +229,14 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
 
     @property
     def _default_params(self):
-        return {
+        default_parameters = {
             "model": self.model_name,
             "max_tokens": self.max_output_tokens,
             "temperature": self.temperature,
             "top_k": self.top_k,
             "top_p": self.top_p,
         }
+        return {**default_parameters, **self.model_kwargs}
 
     def _format_params(
         self,
@@ -234,7 +245,9 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
         stop: Optional[List[str]] = None,
         **kwargs: Any,
     ) -> Dict[str, Any]:
-        system_message, formatted_messages = _format_messages_anthropic(messages)
+        system_message, formatted_messages = _format_messages_anthropic(
+            messages, self.project
+        )
         params = self._default_params
         params.update(kwargs)
         if kwargs.get("model_name"):
@@ -267,14 +280,8 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
             )
         else:
             msg = AIMessage(content=content)
-        # Collect token usage
-        msg.usage_metadata = CacheUsageMetadata(
-            input_tokens=data.usage.input_tokens,
-            output_tokens=data.usage.output_tokens,
-            total_tokens=data.usage.input_tokens + data.usage.output_tokens,
-            cache_creation_input_tokens=data.usage.cache_creation_input_tokens,
-            cache_read_input_tokens=data.usage.cache_read_input_tokens,
-        )
+        # Collect token usage using the reusable function (matches langchain_anthropic)
+        msg.usage_metadata = _create_usage_metadata(data.usage)
         return ChatResult(
             generations=[ChatGeneration(message=msg)],
             llm_output=llm_output,
@@ -294,7 +301,11 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
                 messages, stop=stop, run_manager=run_manager, **kwargs
             )
             return generate_from_stream(stream_iter)
-        retry_decorator = _create_retry_decorator(self, run_manager=run_manager)
+        retry_decorator = _create_retry_decorator(
+            max_retries=self.max_retries,
+            run_manager=run_manager,
+            wait_exponential_kwargs=self.wait_exponential_kwargs,
+        )
 
         @retry_decorator
         def _completion_with_retry_inner(**params: Any) -> Any:
@@ -317,7 +328,11 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
                 messages, stop=stop, run_manager=run_manager, **kwargs
             )
             return await agenerate_from_stream(stream_iter)
-        retry_decorator = _create_retry_decorator(self, run_manager=run_manager)
+        retry_decorator = _create_retry_decorator(
+            max_retries=self.max_retries,
+            run_manager=run_manager,
+            wait_exponential_kwargs=self.wait_exponential_kwargs,
+        )
 
         @retry_decorator
         async def _acompletion_with_retry_inner(**params: Any) -> Any:
@@ -343,14 +358,23 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
         if stream_usage is None:
             stream_usage = self.stream_usage
         params = self._format_params(messages=messages, stop=stop, **kwargs)
-        retry_decorator = _create_retry_decorator(self, run_manager=run_manager)
+        retry_decorator = _create_retry_decorator(
+            max_retries=self.max_retries,
+            run_manager=run_manager,
+            wait_exponential_kwargs=self.wait_exponential_kwargs,
+        )
 
         @retry_decorator
         def _stream_with_retry(**params: Any) -> Any:
+            params.pop("stream", None)
             return self.client.messages.create(**params, stream=True)
 
         stream = _stream_with_retry(**params)
-        coerce_content_to_string = not _tools_in_params(params)
+        coerce_content_to_string = (
+            not _tools_in_params(params)
+            and not _documents_in_params(params)
+            and not _thinking_in_params(params)
+        )
         for event in stream:
             msg = _make_message_chunk_from_anthropic_event(
                 event,
@@ -375,14 +399,23 @@ class ChatAnthropicVertex(_VertexAICommon, BaseChatModel):
         if stream_usage is None:
             stream_usage = self.stream_usage
         params = self._format_params(messages=messages, stop=stop, **kwargs)
-        retry_decorator = _create_retry_decorator(self, run_manager=run_manager)
+        retry_decorator = _create_retry_decorator(
+            max_retries=self.max_retries,
+            run_manager=run_manager,
+            wait_exponential_kwargs=self.wait_exponential_kwargs,
+        )
 
         @retry_decorator
         async def _astream_with_retry(**params: Any) -> Any:
+            params.pop("stream", None)
             return await self.async_client.messages.create(**params, stream=True)
 
         stream = await _astream_with_retry(**params)
-        coerce_content_to_string = not _tools_in_params(params)
+        coerce_content_to_string = (
+            not _tools_in_params(params)
+            and not _documents_in_params(params)
+            and not _thinking_in_params(params)
+        )
         async for event in stream:
             msg = _make_message_chunk_from_anthropic_event(
                 event,
